@@ -755,7 +755,7 @@ Enquire::Internal::get_matching_terms(Xapian::docid did) const
 {
     if (query.empty())
 	throw Xapian::InvalidArgumentError("get_matching_terms with empty query");
-	//return TermIterator(NULL);
+	//return TermIterator();
 
     // The ordered list of terms in the query.
     TermIterator qt = query.get_terms_begin();
@@ -848,12 +848,6 @@ Enquire::Internal::read_doc(const Xapian::Internal::MSetItem &item) const
     }
 }
 
-void
-Enquire::Internal::register_match_decider(const string &,
-	const MatchDecider *)
-{
-}
-
 // Methods of Xapian::Enquire
 
 Enquire::Enquire(const Enquire & other) : internal(other.internal)
