@@ -32,36 +32,38 @@ using namespace std;
 namespace Xapian {
 
 Database
-Remote::open(const string &host, unsigned int port, Xapian::timeout timeout,
+Remote::open(const string &host, unsigned int port, Xapian::timeout timeout_,
 	     Xapian::timeout connect_timeout)
 {
-    LOGCALL_STATIC(API, Database, "Remote::open", host | port | timeout | connect_timeout);
-    return Database(new RemoteTcpClient(host, port, timeout * 1e-3,
+    LOGCALL_STATIC(API, Database, "Remote::open", host | port | timeout_ | connect_timeout);
+    return Database(new RemoteTcpClient(host, port, timeout_ * 1e-3,
 					connect_timeout * 1e-3, false));
 }
 
 WritableDatabase
 Remote::open_writable(const string &host, unsigned int port,
-		      Xapian::timeout timeout, Xapian::timeout connect_timeout)
+		      Xapian::timeout timeout_, Xapian::timeout connect_timeout)
 {
-    LOGCALL_STATIC(API, WritableDatabase, "Remote::open_writable", host | port | timeout | connect_timeout);
-    return WritableDatabase(new RemoteTcpClient(host, port, timeout * 1e-3,
-			    connect_timeout * 1e-3, true));
+    LOGCALL_STATIC(API, WritableDatabase, "Remote::open_writable", host | port | timeout_ | connect_timeout);
+    return WritableDatabase(new RemoteTcpClient(host, port, timeout_ * 1e-3,
+						connect_timeout * 1e-3, true));
 }
 
 Database
-Remote::open(const string &program, const string &args, Xapian::timeout timeout)
+Remote::open(const string &program, const string &args,
+	     Xapian::timeout timeout_)
 {
-    LOGCALL_STATIC(API, Database, "Remote::open", program | args | timeout);
-    return Database(new ProgClient(program, args, timeout * 1e-3, false));
+    LOGCALL_STATIC(API, Database, "Remote::open", program | args | timeout_);
+    return Database(new ProgClient(program, args, timeout_ * 1e-3, false));
 }
 
 WritableDatabase
 Remote::open_writable(const string &program, const string &args,
-		      Xapian::timeout timeout)
+		      Xapian::timeout timeout_)
 {
-    LOGCALL_STATIC(API, WritableDatabase, "Remote::open_writable", program | args | timeout);
-    return WritableDatabase(new ProgClient(program, args, timeout * 1e-3, true));
+    LOGCALL_STATIC(API, WritableDatabase, "Remote::open_writable", program | args | timeout_);
+    return WritableDatabase(new ProgClient(program, args,
+					   timeout_ * 1e-3, true));
 }
 
 }
