@@ -1,8 +1,7 @@
 /* inmemory_positionlist.h
  *
- * ----START-LICENCE----
  * Copyright 1999,2000,2001 BrightStation PLC
- * Copyright 2003 Olly Betts
+ * Copyright 2003,2008,2009 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,9 +15,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
- * -----END-LICENCE-----
  */
 
 #ifndef OM_HGUARD_INMEMORY_POSITIONLIST_H
@@ -33,7 +31,7 @@
 
 using namespace std;
 
-/** A position list in a inemory database. */
+/** A position list in a inmemory database. */
 class InMemoryPositionList : public PositionList
 {
     private:
@@ -56,12 +54,13 @@ class InMemoryPositionList : public PositionList
 	/// Default constructor.
 	InMemoryPositionList() : iterating_in_progress(false) { }
 
+	/// Construct an empty InMemoryPositionList.
+	InMemoryPositionList(bool)
+	    : mypos(positions.begin()), iterating_in_progress(false) { }
+
 	/// Construct, fill list with data, and move the position to the start.
 	InMemoryPositionList(const OmDocumentTerm::term_positions & positions_);
 
-	/// Destructor.
-	~InMemoryPositionList() { }
-
 	/// Fill list with data, and move the position to the start.
 	void set_data(const OmDocumentTerm::term_positions & positions_);
 
