@@ -1,7 +1,8 @@
 /** @file multiandpostlist.cc
  * @brief N-way AND postlist
  */
-/* Copyright (C) 2007 Olly Betts
+/* Copyright (C) 2007,2009 Olly Betts
+ * Copyright (C) 2009 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,6 +23,7 @@
 
 #include "multiandpostlist.h"
 #include "omassert.h"
+#include "debuglog.h"
 
 void
 MultiAndPostList::allocate_plist_and_max_wt()
@@ -97,6 +99,39 @@ MultiAndPostList::get_termfreq_est() const
     return static_cast<Xapian::doccount>(result + 0.5);
 }
 
+TermFreqs
+MultiAndPostList::get_termfreq_est_using_stats(
+	const Xapian::Weight::Internal & stats) const 
+{
+    LOGCALL(MATCH, TermFreqs, "MultiAndPostList::get_termfreq_est_using_stats", stats);
+    // We calculate the estimate assuming independence.  With this assumption,
+    // the estimate is the product of the estimates for the sub-postlists
+    // divided by db_size (n_kids - 1) times.
+    TermFreqs freqs(plist[0]->get_termfreq_est_using_stats(stats));
+
+    double freqest = double(freqs.termfreq);
+    double relfreqest = double(freqs.reltermfreq);
+
+    // Our caller should have ensured this.
+    Assert(stats.collection_size);
+
+    for (size_t i = 1; i < n_kids; ++i) {
+	freqs = plist[i]->get_termfreq_est_using_stats(stats);
+
+	// If the collection is empty, freqest should be 0 already, so leave
+	// it alone.
+	freqest = (freqest * freqs.termfreq) / stats.collection_size;
+
+	// If the rset is empty, relfreqest should be 0 already, so leave
+	// it alone.
+	if (stats.rset_size != 0)
+	    relfreqest = (relfreqest * freqs.reltermfreq) / stats.rset_size;
+    }
+
+    RETURN(TermFreqs(static_cast<Xapian::doccount>(freqest + 0.5),
+		     static_cast<Xapian::doccount>(relfreqest + 0.5)));
+}
+
 Xapian::weight
 MultiAndPostList::get_maxweight() const
 {
@@ -109,13 +144,13 @@ MultiAndPostList::get_docid() const
     return did;
 }
 
-Xapian::doclength
+Xapian::termcount
 MultiAndPostList::get_doclength() const
 {
     Assert(did);
-    Xapian::doclength doclength = plist[0]->get_doclength();
+    Xapian::termcount doclength = plist[0]->get_doclength();
     for (size_t i = 1; i < n_kids; ++i) {
-	AssertEqDouble(doclength, plist[i]->get_doclength());
+	AssertEq(doclength, plist[i]->get_doclength());
     }
     return doclength;
 }
@@ -214,3 +249,13 @@ MultiAndPostList::get_wdf() const
     }
     return totwdf;
 }
+
+Xapian::termcount
+MultiAndPostList::count_matching_subqs() const
+{
+    Xapian::termcount total = 0;
+    for (size_t i = 0; i < n_kids; ++i) {
+	total += plist[i]->count_matching_subqs();
+    }
+    return total;
+}
