@@ -2,8 +2,8 @@
  *
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2009 Olly Betts
- * Copyright 2006,2007 Lemur Consulting Ltd
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009 Olly Betts
+ * Copyright 2006,2007,2008,2009 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -31,8 +31,6 @@
 #include <string>
 #include <vector>
 
-// We have to use the deprecated Quartz::open() method.
-#define XAPIAN_DEPRECATED(D) D
 #include <xapian.h>
 
 #include "backendmanager.h"
@@ -44,8 +42,6 @@
 
 #include "apitest.h"
 
-#include <list>
-
 using namespace std;
 
 static Xapian::Query
@@ -71,34 +67,161 @@ DEFINE_TESTCASE(termstats, backend) {
     return true;
 }
 
-// check that stubdbs work
-DEFINE_TESTCASE(stubdb1, flint) {
-    {
-	// Create the database needed; this is why we require the flint backend.
-	(void) get_database("apitest_simpledata");
-    }
-    ofstream out("stubdb1");
+// Check that stub databases work.
+DEFINE_TESTCASE(stubdb1, backend && !inmemory && !remote) {
+    // Only works for backends which have a path.
+    mkdir(".stub", 0755);
+    const char * dbpath = ".stub/stubdb1";
+    ofstream out(dbpath);
     TEST(out.is_open());
-    // FIXME: not very reliable...
-    out << "remote :" << BackendManager::get_xapian_progsrv_command()
-	<< " .flint/db=apitest_simpledata\n";
+    out << "auto ../" << get_database_path("apitest_simpledata") << endl;
     out.close();
 
     {
-	Xapian::Database db = Xapian::Auto::open_stub("stubdb1");
+	Xapian::Database db = Xapian::Auto::open_stub(dbpath);
 	Xapian::Enquire enquire(db);
 	enquire.set_query(Xapian::Query("word"));
 	enquire.get_mset(0, 10);
     }
     {
-	Xapian::Database db("stubdb1");
+	Xapian::Database db(dbpath);
 	Xapian::Enquire enquire(db);
 	enquire.set_query(Xapian::Query("word"));
 	enquire.get_mset(0, 10);
     }
 
-    unlink("stubdb1");
+    return true;
+}
 
+// Check that stub databases work remotely.
+DEFINE_TESTCASE(stubdb2, backend && !inmemory && !remote) {
+    // Only works for backends which have a path.
+    mkdir(".stub", 0755);
+    const char * dbpath = ".stub/stubdb2";
+    ofstream out(dbpath);
+    TEST(out.is_open());
+    out << "remote :" << BackendManager::get_xapian_progsrv_command()
+	<< ' ' << get_database_path("apitest_simpledata") << endl;
+    out.close();
+
+    {
+	Xapian::Database db = Xapian::Auto::open_stub(dbpath);
+	Xapian::Enquire enquire(db);
+	enquire.set_query(Xapian::Query("word"));
+	enquire.get_mset(0, 10);
+    }
+    {
+	Xapian::Database db(dbpath);
+	Xapian::Enquire enquire(db);
+	enquire.set_query(Xapian::Query("word"));
+	enquire.get_mset(0, 10);
+    }
+
+    return true;
+}
+
+// Regression test - bad entries were ignored after a good entry prior to 1.0.8.
+DEFINE_TESTCASE(stubdb3, backend && !inmemory && !remote) {
+    // Only works for backends which have a path.
+    mkdir(".stub", 0755);
+    const char * dbpath = ".stub/stubdb3";
+    ofstream out(dbpath);
+    TEST(out.is_open());
+    out << "auto ../" << get_database_path("apitest_simpledata") << "\n"
+	   "bad line here\n";
+    out.close();
+
+    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
+	Xapian::Database db = Xapian::Auto::open_stub(dbpath));
+
+    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
+	Xapian::Database db(dbpath));
+
+    return true;
+}
+
+// Test a stub database with just a bad entry.
+DEFINE_TESTCASE(stubdb4, backend && !inmemory && !remote) {
+    // Only works for backends which have a path.
+    mkdir(".stub", 0755);
+    const char * dbpath = ".stub/stubdb4";
+    ofstream out(dbpath);
+    TEST(out.is_open());
+    out << "bad line here\n";
+    out.close();
+
+    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
+	Xapian::Database db = Xapian::Auto::open_stub(dbpath));
+
+    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
+	Xapian::Database db(dbpath));
+
+    return true;
+}
+
+// Test a stub database with a bad entry with no spaces (prior to 1.1.0 this
+// was deliberately allowed, though not documented.
+DEFINE_TESTCASE(stubdb5, backend && !inmemory && !remote) {
+    // Only works for backends which have a path.
+    mkdir(".stub", 0755);
+    const char * dbpath = ".stub/stubdb5";
+    ofstream out(dbpath);
+    TEST(out.is_open());
+    out << "bad\n"
+	   "auto ../" << get_database_path("apitest_simpledata") << endl;
+    out.close();
+
+    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
+	Xapian::Database db = Xapian::Auto::open_stub(dbpath));
+
+    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
+	Xapian::Database db(dbpath));
+
+    return true;
+}
+
+// Test a stub database with an inmemory database (new feature in 1.1.0).
+DEFINE_TESTCASE(stubdb6, inmemory) {
+    mkdir(".stub", 0755);
+    const char * dbpath = ".stub/stubdb6";
+    ofstream out(dbpath);
+    TEST(out.is_open());
+    out << "inmemory\n";
+    out.close();
+
+    // Read-only tests:
+    {
+	Xapian::Database db = Xapian::Auto::open_stub(dbpath);
+	TEST_EQUAL(db.get_doccount(), 0);
+	Xapian::Enquire enquire(db);
+	enquire.set_query(Xapian::Query("word"));
+	Xapian::MSet mset = enquire.get_mset(0, 10);
+	TEST(mset.empty());
+    }
+    {
+	Xapian::Database db(dbpath);
+	TEST_EQUAL(db.get_doccount(), 0);
+	Xapian::Enquire enquire(db);
+	enquire.set_query(Xapian::Query("word"));
+	Xapian::MSet mset = enquire.get_mset(0, 10);
+	TEST(mset.empty());
+    }
+
+    // Writable tests:
+    {
+	Xapian::WritableDatabase db;
+        db = Xapian::Auto::open_stub(dbpath, Xapian::DB_OPEN);
+	TEST_EQUAL(db.get_doccount(), 0);
+	db.add_document(Xapian::Document());
+	TEST_EQUAL(db.get_doccount(), 1);
+    }
+    {
+	Xapian::WritableDatabase db(dbpath, Xapian::DB_OPEN);
+	TEST_EQUAL(db.get_doccount(), 0);
+	db.add_document(Xapian::Document());
+	TEST_EQUAL(db.get_doccount(), 1);
+    }
+ 
     return true;
 }
 
@@ -271,6 +394,9 @@ DEFINE_TESTCASE(matchdecider1, backend && !remote) {
     TEST_EQUAL(mymset.get_matches_lower_bound(), 3);
     TEST_EQUAL(mymset.get_matches_upper_bound(), 3);
     TEST_EQUAL(mymset.get_matches_estimated(), 3);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_lower_bound(), 3);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_upper_bound(), 3);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_estimated(), 3);
     for ( ; i != mymset.end(); ++i) {
 	const Xapian::Document doc(i.get_document());
 	TEST(myfunctor(doc));
@@ -288,20 +414,30 @@ DEFINE_TESTCASE(matchdecider1, backend && !remote) {
     TEST_EQUAL(mymset.size(), 0);
     TEST_EQUAL(mymset.get_matches_lower_bound(), 0);
     TEST_EQUAL(mymset.get_matches_upper_bound(), 6);
-    TEST(mymset.get_matches_estimated() > 0);
-    TEST(mymset.get_matches_estimated() <= 6);
+    TEST_REL(mymset.get_matches_estimated(),>,0);
+    TEST_REL(mymset.get_matches_estimated(),<=,6);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_lower_bound(), 0);
+    TEST_EQUAL(mymset.get_uncollapsed_matches_upper_bound(), 6);
+    TEST_REL(mymset.get_uncollapsed_matches_estimated(),>,0);
+    TEST_REL(mymset.get_uncollapsed_matches_estimated(),<=,6);
 
     // Check that the bounds are appropriate if we ask for only one hit.
     // (Regression test - until SVN 10256, we didn't reduce the lower_bound
     // appropriately, and returned 6 here.)
     mymset = enquire.get_mset(0, 1, 0, &myfunctor);
     TEST_EQUAL(mymset.size(), 1);
-    TEST(mymset.get_matches_lower_bound() >= 1);
-    TEST(mymset.get_matches_lower_bound() <= 3);
-    TEST(mymset.get_matches_upper_bound() >= 3);
-    TEST(mymset.get_matches_upper_bound() <= 6);
-    TEST(mymset.get_matches_estimated() > 0);
-    TEST(mymset.get_matches_estimated() <= 6);
+    TEST_REL(mymset.get_matches_lower_bound(),>=,1);
+    TEST_REL(mymset.get_matches_lower_bound(),<=,3);
+    TEST_REL(mymset.get_matches_upper_bound(),>=,3);
+    TEST_REL(mymset.get_matches_upper_bound(),<=,6);
+    TEST_REL(mymset.get_matches_estimated(),>,0);
+    TEST_REL(mymset.get_matches_estimated(),<=,6);
+    TEST_REL(mymset.get_uncollapsed_matches_lower_bound(),>=,1);
+    TEST_REL(mymset.get_uncollapsed_matches_lower_bound(),<=,3);
+    TEST_REL(mymset.get_uncollapsed_matches_upper_bound(),>=,3);
+    TEST_REL(mymset.get_uncollapsed_matches_upper_bound(),<=,6);
+    TEST_REL(mymset.get_uncollapsed_matches_estimated(),>,0);
+    TEST_REL(mymset.get_uncollapsed_matches_estimated(),<=,6);
 
     // Check that the other documents don't satisfy the condition.
     for (Xapian::docid did = 1; did < docid_checked.size(); ++did) {
@@ -315,12 +451,18 @@ DEFINE_TESTCASE(matchdecider1, backend && !remote) {
     enquire.set_collapse_key(99);
     mymset = enquire.get_mset(0, 1, 0, &myfunctor);
     TEST_EQUAL(mymset.size(), 1);
-    TEST(mymset.get_matches_lower_bound() >= 1);
-    TEST(mymset.get_matches_lower_bound() <= 3);
-    TEST(mymset.get_matches_upper_bound() >= 3);
-    TEST(mymset.get_matches_upper_bound() <= 6);
-    TEST(mymset.get_matches_estimated() > 0);
-    TEST(mymset.get_matches_estimated() <= 6);
+    TEST_REL(mymset.get_matches_lower_bound(),>=,1);
+    TEST_REL(mymset.get_matches_lower_bound(),<=,3);
+    TEST_REL(mymset.get_matches_upper_bound(),>=,3);
+    TEST_REL(mymset.get_matches_upper_bound(),<=,6);
+    TEST_REL(mymset.get_matches_estimated(),>,0);
+    TEST_REL(mymset.get_matches_estimated(),<=,6);
+    TEST_REL(mymset.get_uncollapsed_matches_lower_bound(),>=,1);
+    TEST_REL(mymset.get_uncollapsed_matches_lower_bound(),<=,3);
+    TEST_REL(mymset.get_uncollapsed_matches_upper_bound(),>=,3);
+    TEST_REL(mymset.get_uncollapsed_matches_upper_bound(),<=,6);
+    TEST_REL(mymset.get_uncollapsed_matches_estimated(),>,0);
+    TEST_REL(mymset.get_uncollapsed_matches_estimated(),<=,6);
 
     // Check that the bounds are appropriate if a percentage cutoff is in
     // use.  Set a 1% threshold so we don't actually discard anything.
@@ -328,23 +470,35 @@ DEFINE_TESTCASE(matchdecider1, backend && !remote) {
     enquire.set_cutoff(1);
     mymset = enquire.get_mset(0, 1, 0, &myfunctor);
     TEST_EQUAL(mymset.size(), 1);
-    TEST(mymset.get_matches_lower_bound() >= 1);
-    TEST(mymset.get_matches_lower_bound() <= 3);
-    TEST(mymset.get_matches_upper_bound() >= 3);
-    TEST(mymset.get_matches_upper_bound() <= 6);
-    TEST(mymset.get_matches_estimated() > 0);
-    TEST(mymset.get_matches_estimated() <= 6);
+    TEST_REL(mymset.get_matches_lower_bound(),>=,1);
+    TEST_REL(mymset.get_matches_lower_bound(),<=,3);
+    TEST_REL(mymset.get_matches_upper_bound(),>=,3);
+    TEST_REL(mymset.get_matches_upper_bound(),<=,6);
+    TEST_REL(mymset.get_matches_estimated(),>,0);
+    TEST_REL(mymset.get_matches_estimated(),<=,6);
+    TEST_REL(mymset.get_uncollapsed_matches_lower_bound(),>=,1);
+    TEST_REL(mymset.get_uncollapsed_matches_lower_bound(),<=,3);
+    TEST_REL(mymset.get_uncollapsed_matches_upper_bound(),>=,3);
+    TEST_REL(mymset.get_uncollapsed_matches_upper_bound(),<=,6);
+    TEST_REL(mymset.get_uncollapsed_matches_estimated(),>,0);
+    TEST_REL(mymset.get_uncollapsed_matches_estimated(),<=,6);
 
     // And now with both a collapse key and percentage cutoff.
     enquire.set_collapse_key(99);
     mymset = enquire.get_mset(0, 1, 0, &myfunctor);
     TEST_EQUAL(mymset.size(), 1);
-    TEST(mymset.get_matches_lower_bound() >= 1);
-    TEST(mymset.get_matches_lower_bound() <= 3);
-    TEST(mymset.get_matches_upper_bound() >= 3);
-    TEST(mymset.get_matches_upper_bound() <= 6);
-    TEST(mymset.get_matches_estimated() > 0);
-    TEST(mymset.get_matches_estimated() <= 6);
+    TEST_REL(mymset.get_matches_lower_bound(),>=,1);
+    TEST_REL(mymset.get_matches_lower_bound(),<=,3);
+    TEST_REL(mymset.get_matches_upper_bound(),>=,3);
+    TEST_REL(mymset.get_matches_upper_bound(),<=,6);
+    TEST_REL(mymset.get_matches_estimated(),>,0);
+    TEST_REL(mymset.get_matches_estimated(),<=,6);
+    TEST_REL(mymset.get_uncollapsed_matches_lower_bound(),>=,1);
+    TEST_REL(mymset.get_uncollapsed_matches_lower_bound(),<=,3);
+    TEST_REL(mymset.get_uncollapsed_matches_upper_bound(),>=,3);
+    TEST_REL(mymset.get_uncollapsed_matches_upper_bound(),<=,6);
+    TEST_REL(mymset.get_uncollapsed_matches_estimated(),>,0);
+    TEST_REL(mymset.get_uncollapsed_matches_estimated(),<=,6);
 
     return true;
 }
@@ -401,26 +555,35 @@ DEFINE_TESTCASE(matchdecider3, backend && !remote) {
 
     myMatchDecider2 myfunctor;
 
-    Xapian::MSet mymset1 = enquire.get_mset(0, 2, 0, NULL, &myfunctor);
-    Xapian::MSet mymset2 = enquire.get_mset(0, 1000, 0, NULL, &myfunctor);
+    Xapian::MSet mset1 = enquire.get_mset(0, 2, 0, NULL, &myfunctor);
+    Xapian::MSet mset2 = enquire.get_mset(0, 1000, 0, NULL, &myfunctor);
 
-    // mymset2 should contain all the hits, so the statistics should be exact.
-    TEST_EQUAL(mymset2.get_matches_estimated(),
-	       mymset2.size());
-    TEST_EQUAL(mymset2.get_matches_lower_bound(),
-	       mymset2.get_matches_estimated());
-    TEST_EQUAL(mymset2.get_matches_estimated(),
-	       mymset2.get_matches_upper_bound());
+    // mset2 should contain all the hits, so the statistics should be exact.
+    TEST_EQUAL(mset2.get_matches_estimated(), mset2.size());
+    TEST_EQUAL(mset2.get_matches_lower_bound(), mset2.get_matches_estimated());
+    TEST_EQUAL(mset2.get_matches_estimated(), mset2.get_matches_upper_bound());
 
-    // Check that the lower bound in mymset1 is not greater than the known
+    TEST_REL(mset2.get_uncollapsed_matches_lower_bound(),<=,mset2.get_uncollapsed_matches_estimated());
+    TEST_REL(mset2.get_uncollapsed_matches_estimated(),<=,mset2.get_uncollapsed_matches_upper_bound());
+
+    // Check that the lower bound in mset1 is not greater than the known
     // number of hits.  This failed until revision 10811.
-    TEST_LESSER_OR_EQUAL(mymset1.get_matches_lower_bound(),
-			 mymset2.size());
+    TEST_REL(mset1.get_matches_lower_bound(),<=,mset2.size());
 
-    // Check that the bounds for mymset1 make sense
-    TEST_LESSER_OR_EQUAL(mymset1.get_matches_lower_bound(), mymset1.get_matches_estimated());
-    TEST_LESSER_OR_EQUAL(mymset1.size(), mymset1.get_matches_upper_bound());
-    TEST_LESSER_OR_EQUAL(mymset1.get_matches_estimated(), mymset1.get_matches_upper_bound());
+    // Check that the bounds for mset1 make sense.
+    TEST_REL(mset1.get_matches_lower_bound(),<=,mset1.get_matches_estimated());
+    TEST_REL(mset1.get_matches_estimated(),<=,mset1.get_matches_upper_bound());
+    TEST_REL(mset1.size(),<=,mset1.get_matches_upper_bound());
+
+    TEST_REL(mset1.get_uncollapsed_matches_lower_bound(),<=,mset1.get_uncollapsed_matches_estimated());
+    TEST_REL(mset1.get_uncollapsed_matches_estimated(),<=,mset1.get_uncollapsed_matches_upper_bound());
+
+    // The uncollapsed match would match all documents but the one the
+    // matchdecider rejects.
+    TEST_REL(mset1.get_uncollapsed_matches_upper_bound(),>=,db.get_doccount() - 1);
+    TEST_REL(mset1.get_uncollapsed_matches_upper_bound(),<=,db.get_doccount());
+    TEST_REL(mset2.get_uncollapsed_matches_upper_bound(),>=,db.get_doccount() - 1);
+    TEST_REL(mset2.get_uncollapsed_matches_upper_bound(),<=,db.get_doccount());
 
     return true;
 }
@@ -618,7 +781,7 @@ DEFINE_TESTCASE(collapsekey2, backend) {
     Xapian::Enquire enquire(get_database("apitest_simpledata2"));
     enquire.set_query(Xapian::Query("this"));
 
-    Xapian::MSet mymset1 = enquire.get_mset(0, 1);
+    Xapian::MSet mset1 = enquire.get_mset(0, 1);
 
     // Test that if no duplicates are found, then the upper bound remains
     // unchanged and the lower bound drops.
@@ -626,10 +789,10 @@ DEFINE_TESTCASE(collapsekey2, backend) {
 	enquire.set_query(Xapian::Query("this"));
 	Xapian::valueno value_no = 3;
 	enquire.set_collapse_key(value_no);
-	Xapian::MSet mymset = enquire.get_mset(0, 1);
+	Xapian::MSet mset = enquire.get_mset(0, 1);
 
-	TEST(mymset.get_matches_lower_bound() < mymset1.get_matches_lower_bound());
-	TEST_EQUAL(mymset.get_matches_upper_bound(), mymset1.get_matches_upper_bound());
+	TEST_REL(mset.get_matches_lower_bound(),<,mset1.get_matches_lower_bound());
+	TEST_EQUAL(mset.get_matches_upper_bound(), mset1.get_matches_upper_bound());
     }
 
     return true;
@@ -952,7 +1115,8 @@ DEFINE_TESTCASE(specialterms1, backend) {
     return true;
 }
 
-// test that terms with a special characters in appear correctly when iterating allterms
+// test that terms with a special characters in appear correctly when iterating
+// allterms
 DEFINE_TESTCASE(specialterms2, backend) {
     Xapian::Database db(get_database("apitest_space"));
 
@@ -1034,8 +1198,7 @@ DEFINE_TESTCASE(multiexpand1, backend && !multi) {
     rset2.add_document(1);
     rset2.add_document(2);
 
-    // retrieve the top ten results from each method of accessing
-    // multiple text files
+    // Retrieve all the ESet results in each of the three setups:
 
     // This is the single database one.
     Xapian::ESet eset1 = enquire1.get_eset(1000, rset1);
@@ -1046,25 +1209,32 @@ DEFINE_TESTCASE(multiexpand1, backend && !multi) {
     // This is the multi database without approximation
     Xapian::ESet eset3 = enquire2.get_eset(1000, rset2, Xapian::Enquire::USE_EXACT_TERMFREQ);
 
-    TEST_EQUAL(eset1.size(), eset2.size());
     TEST_EQUAL(eset1.size(), eset3.size());
 
     Xapian::ESetIterator i = eset1.begin();
-    Xapian::ESetIterator j = eset2.begin();
-    Xapian::ESetIterator k = eset3.begin();
-    bool all_iwts_equal_jwts = true;
-    while (i != eset1.end() && j != eset2.end() && k != eset3.end()) {
-	if (i.get_weight() != j.get_weight()) all_iwts_equal_jwts = false;
-	TEST_EQUAL(i.get_weight(), k.get_weight());
-	TEST_EQUAL(*i, *k);
+    Xapian::ESetIterator j = eset3.begin();
+    while (i != eset1.end() && j != eset3.end()) {
+	TEST_EQUAL(*i, *j);
+	TEST_EQUAL(i.get_weight(), j.get_weight());
 	++i;
 	++j;
-	++k;
     }
     TEST(i == eset1.end());
-    TEST(j == eset2.end());
-    TEST(k == eset3.end());
-    TEST(!all_iwts_equal_jwts);
+    TEST(j == eset3.end());
+
+    bool eset1_eq_eset2 = true;
+    i = eset1.begin();
+    j = eset2.begin();
+    while (i != eset1.end() && j != eset2.end()) {
+	if (i.get_weight() != j.get_weight()) {
+	    eset1_eq_eset2 = false;
+	    break;
+	}
+	++i;
+	++j;
+    }
+    TEST(!eset1_eq_eset2);
+
     return true;
 }
 
@@ -1292,99 +1462,6 @@ DEFINE_TESTCASE(consistency1, backend && !remote) {
     return true;
 }
 
-// tests that specifying a nonexistent input file throws an exception.
-DEFINE_TESTCASE(quartzdatabaseopeningerror1, quartz) {
-    mkdir(".quartz", 0755);
-
-    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
-		   Xapian::Quartz::open(".quartz/nosuchdirectory"));
-    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
-		   Xapian::Quartz::open(".quartz/nosuchdirectory", Xapian::DB_OPEN));
-
-    mkdir(".quartz/emptydirectory", 0700);
-    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
-		   Xapian::Quartz::open(".quartz/emptydirectory"));
-
-    touch(".quartz/somefile");
-    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
-	Xapian::Quartz::open(".quartz/somefile"));
-    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
-	Xapian::Quartz::open(".quartz/somefile", Xapian::DB_OPEN));
-    TEST_EXCEPTION(Xapian::DatabaseCreateError,
-	Xapian::Quartz::open(".quartz/somefile", Xapian::DB_CREATE));
-    TEST_EXCEPTION(Xapian::DatabaseCreateError,
-	Xapian::Quartz::open(".quartz/somefile", Xapian::DB_CREATE_OR_OPEN));
-    TEST_EXCEPTION(Xapian::DatabaseCreateError,
-	Xapian::Quartz::open(".quartz/somefile", Xapian::DB_CREATE_OR_OVERWRITE));
-
-    return true;
-}
-
-/// Test opening of a quartz database
-DEFINE_TESTCASE(quartzdatabaseopen1, quartz) {
-    const char * dbdir = ".quartz/test_quartzdatabaseopen1";
-    mkdir(".quartz", 0755);
-
-    {
-	rm_rf(dbdir);
-	Xapian::WritableDatabase wdb =
-	    Xapian::Quartz::open(dbdir, Xapian::DB_CREATE);
-	TEST_EXCEPTION(Xapian::DatabaseLockError,
-	    Xapian::Quartz::open(dbdir, Xapian::DB_OPEN));
-	Xapian::Quartz::open(dbdir);
-    }
-
-    {
-	rm_rf(dbdir);
-	mkdir(dbdir, 0700);
-	Xapian::WritableDatabase wdb =
-	    Xapian::Quartz::open(dbdir, Xapian::DB_CREATE);
-	TEST_EXCEPTION(Xapian::DatabaseLockError,
-	    Xapian::Quartz::open(dbdir, Xapian::DB_OPEN));
-	Xapian::Quartz::open(dbdir);
-    }
-
-    {
-	rm_rf(dbdir);
-	Xapian::WritableDatabase wdb =
-	    Xapian::Quartz::open(dbdir, Xapian::DB_CREATE_OR_OPEN);
-	TEST_EXCEPTION(Xapian::DatabaseLockError,
-	    Xapian::Quartz::open(dbdir, Xapian::DB_CREATE_OR_OVERWRITE));
-	Xapian::Quartz::open(dbdir);
-    }
-
-    {
-	rm_rf(dbdir);
-	Xapian::WritableDatabase wdb =
-	    Xapian::Quartz::open(dbdir, Xapian::DB_CREATE_OR_OVERWRITE);
-	TEST_EXCEPTION(Xapian::DatabaseLockError,
-	    Xapian::Quartz::open(dbdir, Xapian::DB_CREATE_OR_OPEN));
-	Xapian::Quartz::open(dbdir);
-    }
-
-    {
-	TEST_EXCEPTION(Xapian::DatabaseCreateError,
-	    Xapian::Quartz::open(dbdir, Xapian::DB_CREATE));
-	Xapian::WritableDatabase wdb =
-	    Xapian::Quartz::open(dbdir, Xapian::DB_CREATE_OR_OVERWRITE);
-	Xapian::Quartz::open(dbdir);
-    }
-
-    {
-	Xapian::WritableDatabase wdb =
-	    Xapian::Quartz::open(dbdir, Xapian::DB_CREATE_OR_OPEN);
-	Xapian::Quartz::open(dbdir);
-    }
-
-    {
-	Xapian::WritableDatabase wdb =
-	    Xapian::Quartz::open(dbdir, Xapian::DB_OPEN);
-	Xapian::Quartz::open(dbdir);
-    }
-
-    return true;
-}
-
 // tests that specifying a nonexistent input file throws an exception.
 DEFINE_TESTCASE(flintdatabaseopeningerror1, flint) {
 #ifdef XAPIAN_HAS_FLINT_BACKEND
@@ -1760,7 +1837,12 @@ DEFINE_TESTCASE(netstats1, remote) {
 	Xapian::Enquire enq(db);
 	enq.set_query(query);
 	Xapian::MSet mset = enq.get_mset(0, MSET_SIZE, &rset);
-	TEST_EQUAL(mset, mset_alllocal);
+	TEST_EQUAL(mset.get_matches_lower_bound(), mset_alllocal.get_matches_lower_bound());
+	TEST_EQUAL(mset.get_matches_upper_bound(), mset_alllocal.get_matches_upper_bound());
+	TEST_EQUAL(mset.get_matches_estimated(), mset_alllocal.get_matches_estimated());
+	TEST_EQUAL(mset.get_max_attained(), mset_alllocal.get_max_attained());
+	TEST_EQUAL(mset.size(), mset_alllocal.size());
+	TEST(mset_range_is_same(mset, 0, mset_alllocal, 0, mset.size()));
     }
 
     {
@@ -1771,7 +1853,12 @@ DEFINE_TESTCASE(netstats1, remote) {
 	Xapian::Enquire enq(db);
 	enq.set_query(query);
 	Xapian::MSet mset = enq.get_mset(0, MSET_SIZE, &rset);
-	TEST_EQUAL(mset, mset_alllocal);
+	TEST_EQUAL(mset.get_matches_lower_bound(), mset_alllocal.get_matches_lower_bound());
+	TEST_EQUAL(mset.get_matches_upper_bound(), mset_alllocal.get_matches_upper_bound());
+	TEST_EQUAL(mset.get_matches_estimated(), mset_alllocal.get_matches_estimated());
+	TEST_EQUAL(mset.get_max_attained(), mset_alllocal.get_max_attained());
+	TEST_EQUAL(mset.size(), mset_alllocal.size());
+	TEST(mset_range_is_same(mset, 0, mset_alllocal, 0, mset.size()));
     }
 
     {
@@ -1782,7 +1869,12 @@ DEFINE_TESTCASE(netstats1, remote) {
 	Xapian::Enquire enq(db);
 	enq.set_query(query);
 	Xapian::MSet mset = enq.get_mset(0, MSET_SIZE, &rset);
-	TEST_EQUAL(mset, mset_alllocal);
+	TEST_EQUAL(mset.get_matches_lower_bound(), mset_alllocal.get_matches_lower_bound());
+	TEST_EQUAL(mset.get_matches_upper_bound(), mset_alllocal.get_matches_upper_bound());
+	TEST_EQUAL(mset.get_matches_estimated(), mset_alllocal.get_matches_estimated());
+	TEST_EQUAL(mset.get_max_attained(), mset_alllocal.get_max_attained());
+	TEST_EQUAL(mset.size(), mset_alllocal.size());
+	TEST(mset_range_is_same(mset, 0, mset_alllocal, 0, mset.size()));
     }
 
     return true;
@@ -1790,24 +1882,27 @@ DEFINE_TESTCASE(netstats1, remote) {
 
 // Coordinate matching - scores 1 for each matching term
 class MyWeight : public Xapian::Weight {
-    public:
-	MyWeight * clone() const {
-	    return new MyWeight;
-	}
-	MyWeight() { }
-	~MyWeight() { }
-	std::string name() const { return "Coord"; }
-	std::string serialise() const { return ""; }
-	MyWeight * unserialise(const std::string & /*s*/) const {
-	    return new MyWeight;
-	}
-	Xapian::weight get_sumpart(Xapian::termcount /*wdf*/, Xapian::doclength /*len*/) const { return 1; }
-	Xapian::weight get_maxpart() const { return 1; }
+    double scale_factor;
 
-	Xapian::weight get_sumextra(Xapian::doclength /*len*/) const { return 0; }
-	Xapian::weight get_maxextra() const { return 0; }
+  public:
+    MyWeight * clone() const {
+	return new MyWeight;
+    }
+    void init(double factor) {
+	scale_factor = factor;
+    }
+    MyWeight() { }
+    ~MyWeight() { }
+    std::string name() const { return "MyWeight"; }
+    string serialise() const { return string(); }
+    MyWeight * unserialise(const string &) const { return new MyWeight; }
+    Xapian::weight get_sumpart(Xapian::termcount, Xapian::termcount) const {
+	return scale_factor;
+    }
+    Xapian::weight get_maxpart() const { return scale_factor; }
 
-	bool get_sumpart_needs_doclength() const { return false; }
+    Xapian::weight get_sumextra(Xapian::termcount) const { return 0; }
+    Xapian::weight get_maxextra() const { return 0; }
 };
 
 // tests user weighting scheme.
@@ -1845,13 +1940,14 @@ DEFINE_TESTCASE(matchall1, backend) {
     enquire.set_query(Xapian::Query::MatchAll);
     Xapian::MSet mset = enquire.get_mset(0, 10);
     TEST_EQUAL(mset.get_matches_lower_bound(), db.get_doccount());
-
+    TEST_EQUAL(mset.get_uncollapsed_matches_lower_bound(), db.get_doccount());
 
     enquire.set_query(Xapian::Query(Xapian::Query::OP_OR,
 				    Xapian::Query("nosuchterm"),
 				    Xapian::Query::MatchAll));
     mset = enquire.get_mset(0, 10);
     TEST_EQUAL(mset.get_matches_lower_bound(), db.get_doccount());
+    TEST_EQUAL(mset.get_uncollapsed_matches_lower_bound(), db.get_doccount());
 
     // Check that the results aren't ranked by relevance (fixed in 1.0.9).
     TEST(mset.size() > 1);
@@ -1861,3 +1957,24 @@ DEFINE_TESTCASE(matchall1, backend) {
  
     return true;
 }
+
+// Test using a ValueSetMatchDecider
+DEFINE_TESTCASE(valuesetmatchdecider2, backend && !remote) {
+    Xapian::Database db(get_database("apitest_phrase"));
+    Xapian::Enquire enq(db);
+    enq.set_query(Xapian::Query("leav"));
+
+    Xapian::ValueSetMatchDecider vsmd1(1, true);
+    vsmd1.add_value("n");
+    Xapian::ValueSetMatchDecider vsmd2(1, false);
+    vsmd2.add_value("n");
+
+    Xapian::MSet mymset = enq.get_mset(0, 20);
+    mset_expect_order(mymset, 8, 6, 4, 5, 7, 10, 12, 11, 13, 9, 14);
+    mymset = enq.get_mset(0, 20, 0, NULL, &vsmd1);
+    mset_expect_order(mymset, 6, 12);
+    mymset = enq.get_mset(0, 20, 0, NULL, &vsmd2);
+    mset_expect_order(mymset, 8, 4, 5, 7, 10, 11, 13, 9, 14);
+
+    return true;
+}
